function Y = iqdft(X, A, L)
% IQDFT Inverse discrete quaternion Fourier transform.
%
% This function implements the inverse discrete quaternion Fourier transform of X.
% It does so using direct evaluation using a matrix product, and it is
% intended mainly for comparing results against the fast implementation
% IQFFT. Because this is an inverse transform it divides the result by N, the
% length of the transform.

% Copyright  2005 Stephen J. Sangwine and Nicolas Le Bihan.
% See the file : Copyright.m for further details.

error(nargchk(1, 3, nargin)), error(nargoutchk(0, 1, nargout))

if nargin == 1
    A = quaternion(1, 1, 1); % No values were given for A or L,
    L = 'L';                 % so supply the default values.
end

if nargin == 2 % One of A or L was specified, but we don't know which.
    if A == 'L' | A == 'R'
        L = A;                 % A has a sensible value for L, so copy it
        A = quaternion(1,1,1); % into L and supply a default value for A.
    else
        L = 'L'; % Assume that A is an axis, and supply a default value for L.
    end
end

if size(A) ~= [1, 1]
    error('The transform axis cannot be a matrix or vector.');
end

if ~isa(A, 'quaternion') | ~ispure(A)
    error('The transform axis must be a pure quaternion.')
end

if L ~= 'L' & L ~= 'R'
    error('L must have the value ''L'' or ''R''.');
end

A = unit(A); % Ensure that A is a unit (pure) quaternion.

% Compute the transform. This is done by explicit evaluation of the products
% hypercomplex exponentials with the columns of A. All the hypercomplex
% exponentials are created at once, and the product and summation is done by
% a matrix product.

[r,c] = size(X);

Y = quaternion(zeros(r,c)); % Construct an empty output array.

E = exp(A .* 2 .* pi .* ((0:r-1)' *(0:r-1)) ./r); % A matrix of exponential values.

if L == 'L'
    Y = Y + E * X; % Multiply the exponential matrix on the left.
elseif L == 'R'
    Y = Y + (X.' * E.').'; % To multiply the exponential matrix on the right we
                           % have to transpose both and transpose the result.
else
    error('L has incorrect value');
end

Y = Y ./ r; % Divide by the length of the transform.
