function Y = qfft2(X, A, L)
% QFFT2 Discrete quaternion Fourier transform.
%
% This function implements the discrete quaternion 2D Fourier transform of X.
% A specifies the transform axis (that is the direction in 3-space of the
% vector part of the hypercomplex exponential. It must be a pure quaternion
% but it need not have unit modulus. If omitted, a default axis of (1,1,1) is
% assumed. L specifies whether the exponential is on the left ('L') or right
% ('R'). No other values are accepted, but there is a default of 'L' if no
% value is given.
%
% See also: IQFFT2.

% Copyright  2005 Stephen J. Sangwine and Nicolas Le Bihan.
% See the file : Copyright.m for further details.

error(nargchk(1, 3, nargin)), error(nargoutchk(0, 1, nargout))

if nargin == 1
    A = quaternion(1, 1, 1); % No values were given for A or L,
    L = 'L';                 % so supply the default values.
end

if nargin == 2 % One of A or L was specified, but we don't know which.
    if A == 'L' | A == 'R'
        L = A;                 % A has a sensible value for L, so copy it
        A = quaternion(1,1,1); % into L and supply a default value for A.
    else
        L = 'L'; % Assume that A is an axis, and supply a default value for L.
    end
end

if size(A) ~= [1, 1]
    error('The transform axis cannot be a matrix or vector.');
end

if ~isa(A, 'quaternion') | ~ispure(A)
    error('The transform axis must be a pure quaternion.')
end

if L ~= 'L' & L ~= 'R'
    error('L must have the value ''L'' or ''R''.');
end

S = 1;
if L == 'R'
    S = -1;  % S is a sign bit used to conjugate one of the complex components
             % below when the exponential is on the right.
end

A = unit(A); % Ensure that A is a unit (pure) quaternion.
B = orthonormal_basis(A);

% Compute the transform. This is done by changing the basis of all the elements
% of the matrix or vector X to the transform axis.  The quaternion elements can
% then be regrouped as complex values and their FFTs computed with the standard
% Matlab function.  We then have to regroup the complex results into quaternion
% form and invert the change of basis.

X = change_basis(X, B);

C1 = fft2(complex(scalar(X),      x(X))); % Compute the two complex FFTs using the
C2 = fft2(complex(     y(X), S .* z(X))); % standard Matlab complex FFT function.

Y = quaternion(real(C1),      imag(C1), ... % Compose a quaternion result from
               real(C2), S .* imag(C2));    % the two complex results.

Y = change_basis(Y, B.'); % Change back to the original basis.
